"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.transformChords = exports.getChordTextScale = exports.getChordLength = void 0;
function _typeof(o) { "@babel/helpers - typeof"; return _typeof = "function" == typeof Symbol && "symbol" == typeof Symbol.iterator ? function (o) { return typeof o; } : function (o) { return o && "function" == typeof Symbol && o.constructor === Symbol && o !== Symbol.prototype ? "symbol" : typeof o; }, _typeof(o); }
function ownKeys(e, r) { var t = Object.keys(e); if (Object.getOwnPropertySymbols) { var o = Object.getOwnPropertySymbols(e); r && (o = o.filter(function (r) { return Object.getOwnPropertyDescriptor(e, r).enumerable; })), t.push.apply(t, o); } return t; }
function _objectSpread(e) { for (var r = 1; r < arguments.length; r++) { var t = null != arguments[r] ? arguments[r] : {}; r % 2 ? ownKeys(Object(t), !0).forEach(function (r) { _defineProperty(e, r, t[r]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(e, Object.getOwnPropertyDescriptors(t)) : ownKeys(Object(t)).forEach(function (r) { Object.defineProperty(e, r, Object.getOwnPropertyDescriptor(t, r)); }); } return e; }
function _defineProperty(obj, key, value) { key = _toPropertyKey(key); if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
function _toPropertyKey(t) { var i = _toPrimitive(t, "string"); return "symbol" == _typeof(i) ? i : String(i); }
function _toPrimitive(t, r) { if ("object" != _typeof(t) || !t) return t; var e = t[Symbol.toPrimitive]; if (void 0 !== e) { var i = e.call(t, r || "default"); if ("object" != _typeof(i)) return i; throw new TypeError("@@toPrimitive must return a primitive value."); } return ("string" === r ? String : Number)(t); }
var transformChords = exports.transformChords = function transformChords(bar) {
  var chords = bar.chords.map(function (chord) {
    return !chord.pitch ? chord : _objectSpread(_objectSpread({}, chord), {}, {
      pitch: replaceFlat(chord.pitch),
      chordType: getSpecialSymbol(chord.chordType),
      sup: chord.sup ? replaceFlat(chord.sup.substr(1, chord.sup.length - 2)) : null,
      bass: chord.bass ? replaceFlat(chord.bass.substr(1, chord.bass.length)) : null
    });
  });
  return _objectSpread(_objectSpread({}, bar), {}, {
    chords: chords
  });
};
var replaceFlat = function replaceFlat(str) {
  return str.replace(/b/g, "\u266D");
};
var getSpecialSymbol = function getSpecialSymbol(chordType) {
  if (chordType === "maj7") return "\u2206"; //  ∆
  if (chordType === "halfdim") return "\xF8"; //  ø

  return chordType.replace("dim", "\u2218"); // ∘
};
var getChordTextScale = exports.getChordTextScale = function getChordTextScale(bars, barWidth) {
  /*
  F maj7
  Db maj7
  B halfdim
  F# halfdim
  F 7(sus4,9)
  Eb 7(sus4,9)
  Db major-13th
  Ab 7(#11,9,13)
  Eb 7(sus4,9,13)
  C suspended-fourth
  */
  var standardCharWidth = 17;
  var smallestCharWidth = bars.reduce(function (prev, bar, i) {
    var chordWidth = barWidth / bar.chords.length;
    var maxChorStringLength = getLongestChordLength(bar);
    var charWidth = maxChorStringLength ? chordWidth / maxChorStringLength : 0;
    return !prev ? charWidth : prev > charWidth ? charWidth : prev;
  }, null);
  if (standardCharWidth < smallestCharWidth) return 1;
  return smallestCharWidth / standardCharWidth;
};
var defaultLength = 3.75;
var getChordLength = exports.getChordLength = function getChordLength(_ref) {
  var pitch = _ref.pitch,
    chordType = _ref.chordType,
    sup = _ref.sup;
  return pitch !== null && pitch !== void 0 && pitch.length || 0 + chordType ? defaultLength : 0 + sup ? defaultLength : 0;
};
var getLongestChordLength = function getLongestChordLength(bar) {
  return bar.chords.reduce(function (prev, chord) {
    var chorLength = getChordLength(chord);
    return chorLength > prev ? chorLength : prev;
  }, 0);
};